/*
    Name    :"AXI-Stream FIFO".
    Source  :Written by GowinSynthesis.
    Date    :06/14/2023

    Application:Allows memory mapped access to a AXI4-Stream interface.

    Features:
        1)32bit axi4-lite slave interface;
        2)Axi4-full/lite data interface type can be config;
        3)Configurable 32/64/128/256/512b AXI4-Stream data interface,"Slaver supports 32bit/64bit only for this design";
        4)Independently internal TX and RX data FIFOs(32/64 DATA in,32/64 DATA out);
        5)Full duplex operation;
        6)Supports AXI Ethernet basic mode;
        7)Provides interrupts for error and status conditions;
        8)TX and RX cut-through mode;
        9)a.When AXI4-Lite is selected, register access and FIFO accesses are handled by the AXI4-Lite interface. 
          b.When AXI4-full is selected, register access is handled by AXI4-Lite interface and FIFO accesses are handled by AXI4 interface.

        10)a.For AXI4 Lite Ports: For Strb every byte is valid,that is Strb's every bit is 'b1.
           b.For Stream Ports: For Strb,every byte is valid,that is Strb's every bit is 'b1.
                               Nonsupport Transmit Control.
                               Nonsupport ID,DESET,USER,KEEP.
           c.For AXI4 Ports: For Strb,every byte is valid,that is Strb's every bit is 'b1.
                             Nonsupport ID.
                             Nonsupport Data Width of 128bit,256bit,512bit

    component and module:
  ____________________________________________________________________________________________________________________________________________________________________________________________________
 |                          _______________________________________________________________________________________________________________________________________                                   |
 |                         |       _________________________                                              AXI4 stream fifo                                        |                                   |
 |                         |      |  AXI4_LITE interface    |                ___________                                                                          |                                   |
 |                         |      |       _______           | ------------->| REG space |                                                                         |                                   |
 | AXI4_lite   <----------------->|      |       |          |                ----------                                                                           |                                   |
 |  interface              |      |      | slave |          |       _________________        ___________________________________                                  |                                   |
 |                         |      |      |_______|          |      | AXI4-LITE_Tdata |      |                                   |                                 |                                   |
 |                         |      |      ____________       |  ---------------------------->|                                   |                                 |                                   |
 |                         |      |     | interrupt  |      |     _________________         |               _______             |                                 |                                   |
 |                         |      |     | controller |      |    | AXI4-LITE_Rdata |        |  Tdata_sel   |       | Transmit   |      __________________         |                                   |
 |                         |      |      ------------       | <-----------------------------|  ----------> | Tfifo | ---------->| --->| Transmit Control |------------->AXI4_stream_TXD interface     |
 |                         |      |_________________________|                               |              |_______|            |      ------------------         |                                   |
 |                         |                                                                |                                   |                                 |                                   |
 |                         |        ________________________                                |                                   |                                 |                                   |
 |                         |       |                         |        _________________     |              _______              |                                 |                                   |
 |                         |       |    AXI4_FULL interface  |       | AXI4-FULL_Rdata |    |  Rdata_sel  |       | Recveive    |      __________________         |                                   |
 |                         |       |       _______           |   <--------------------------|  <----------| Rfifo | <---------- | <---| Receive Control  |<--------------AXI4_stream_RXD interface    |  
 |     AXI4  <-------------------->|      |       |          |      _________________       |             |_______|             |      ------------------         |                                   |
 |  interface              |       |      | slave |          |     | AXI4-FULL_Tdata |      |                                   |                                 |                                   |
 |                         |       |      |_______|          |  --------------------------->|                                   |                                 |                                   |
 |                         |       |_________________________|                              |___________________________________|                                 |                                   |
 |                         |______________________________________________________________________________________________________________________________________|                                   |
 |____________________________________________________________________________________________________________________________________________________________________________________________________| 
*/  

`timescale 1ns / 1ns
`include "master_define.v"
module AXI_Stream_FIFO_TB
//module AXI_Stream_FIFO_Top
    (
    input   wire                                        S_axi_aclk
    );
                        
    reg                             hwRstn           ;
    reg[12:0]                       rst_cnt          ;
    wire                            T_frame          ;
    wire                            T_end            ;
    wire                            R_frame          ;
    wire                            R_end            ;
    wire[`ADDR_WE_OE-1:0]           axi_awaddr_in    ;
    wire[`LITE_DATAWD-1:0]          axi_wdata_in     ;
    wire[`ADDR_WE_OE-1:0]    axi_araddr_in    ;
    wire[`ADDR_WE_OE-1:0]    axi4_awaddr_in   ;
    wire[`FULL_DATAWD-1:0]          axi4_wdata_in    ;
    wire[`ADDR_WE_OE-1:0]    axi4_araddr_in   ;
    wire[`ADDR_WE_OE-1:0]                S_axi_araddr                    ;
    wire                                        S_axi_arready                   ;
    wire                                        S_axi_arvalid                   ;
    wire[`ADDR_WE_OE-1:0]                S_axi_awaddr                    ;
    wire                                        S_axi_awready                   ;
    wire                                        S_axi_awvalid                   ;
    wire                                        S_axi_bready                    ;
    wire[1:0]                                   S_axi_bresp                     ;
    wire                                        S_axi_bvalid                    ;
    wire[`LITE_DATAWD-1:0]                S_axi_rdata                     ;
    wire                                        S_axi_rready                    ;
    wire[1:0]                                   S_axi_rresp                     ;                      
    wire                                        S_axi_rvalid                    ;
    wire[`LITE_DATAWD-1:0]                S_axi_wdata                     ;
    wire                                        S_axi_wready                    ;
    wire[`LITE_DATAWD/8-1:0]              S_axi_wstrb                     ;     
    wire                                        S_axi_wvalid                    ;

    reg[`FULL_DATAWD-1:0]           Axi_str_rxd_tdata;    
    wire                            Axi_str_rxd_tlast;
    wire                            Axi_str_rxd_tready;
    reg                             Axi_str_rxd_tvalid;
    wire[`FULL_DATAWD-1:0]          Axi_str_txd_tdata;    
    wire                            Axi_str_txd_tlast;    
    reg                             Axi_str_txd_tready;   
    wire                            Axi_str_txd_tvalid;  
    reg                             Axi_str_rxd_tlast_wait;
    reg                             Axi_str_txd_tvalid_dly1;
    reg[`FULL_DATAWD-1:0]           Axi_str_txd_tdata_dly1;
    reg                             Axi_str_txd_tvalid_dly2;
    reg                             Axi_str_txd_tvalid_dly3;
    reg                             Axi_str_txd_tvalid_dly4;
    reg[`FULL_DATAWD-1:0]           Axi_str_txd_tdata_dly2;
    reg[`FULL_DATAWD-1:0]           Axi_str_txd_tdata_dly3;
    reg                             Axi_str_txd_tlast_dly;

    wire[`ADDR_WE_OE-1:0]                        S_axi4_araddr         ;
    wire[1:0]                                    S_axi4_arburst        ;
    wire[3:0]                                    S_axi4_arcache        ;
    wire[`FULL_DATAWD-1:0]                       S_axi4_arid           ;
    wire[7:0]                                    S_axi4_arlen          ;
    wire                                         S_axi4_arlock         ;
    wire[2:0]                                    S_axi4_arport         ;
    wire                                         S_axi4_arready        ;
    wire[2:0]                                    S_axi4_arsize         ;
    wire                                         S_axi4_arvalid        ;
    wire[`ADDR_WE_OE-1:0]                        S_axi4_awaddr         ;
    wire[1:0]                                    S_axi4_awburst        ;
    wire[3:0]                                    S_axi4_awcache        ;
    wire[`FULL_DATAWD-1:0]                       S_axi4_awid           ;
    wire[7:0]                                    S_axi4_awlen          ;
    wire                                         S_axi4_awlock         ;
    wire[2:0]                                    S_axi4_awport         ;
    wire                                         S_axi4_awready        ;
    wire[2:0]                                    S_axi4_awsize         ;
    wire                                         S_axi4_awvalid        ;
    wire[`FULL_DATAWD-1:0]                       S_axi4_bid            ;
    wire                                         S_axi4_bready         ;
    wire[1:0]                                    S_axi4_bresp          ;
    wire                                         S_axi4_bvalid         ;
    wire[`FULL_DATAWD-1:0]                       S_axi4_rdata          ;
    wire[`FULL_DATAWD-1:0]                       S_axi4_rid            ;
    wire                                         S_axi4_rlast          ;
    wire                                         S_axi4_rready         ;
    wire[1:0]                                    S_axi4_rresp          ;
    wire                                         S_axi4_rvalid         ;
    wire[`FULL_DATAWD-1:0]                       S_axi4_wdata          ;
    wire                                         S_axi4_wlast          ;
    wire                                         S_axi4_wready         ;
    wire[`FULL_DATAWD/8-1:0]                     S_axi4_wstrb          ;
    wire                                         S_axi4_wvalid         ;
    
AXI_Stream_FIFO_Top AXI_Stream_FIFO_Top_AXI4_Lite(
		.S_axi_aclk(S_axi_aclk), //input S_axi_aclk
		.S_axi_aresetn(hwRstn), //input S_axi_aresetn
		.S_axi_araddr(S_axi_araddr), //input [7:0] S_axi_araddr
		.S_axi_arready(S_axi_arready), //output S_axi_arready
		.S_axi_arvalid(S_axi_arvalid), //input S_axi_arvalid
		.S_axi_awaddr(S_axi_awaddr), //input [7:0] S_axi_awaddr
		.S_axi_awready(S_axi_awready), //output S_axi_awready
		.S_axi_awvalid(S_axi_awvalid), //input S_axi_awvalid
		.S_axi_bready(S_axi_bready), //input S_axi_bready
		.S_axi_bresp(S_axi_bresp), //output [1:0] S_axi_bresp
		.S_axi_bvalid(S_axi_bvalid), //output S_axi_bvalid
		.S_axi_rdata(S_axi_rdata), //output [31:0] S_axi_rdata
		.S_axi_rready(S_axi_rready), //input S_axi_rready
		.S_axi_rresp(S_axi_rresp), //output [1:0] S_axi_rresp
		.S_axi_rvalid(S_axi_rvalid), //output S_axi_rvalid
		.S_axi_wdata(S_axi_wdata), //input [31:0] S_axi_wdata
		.S_axi_wready(S_axi_wready), //output S_axi_wready
		.S_axi_wstrb(4'b1111), //input [3:0] S_axi_wstrb
		.S_axi_wvalid(S_axi_wvalid), //input S_axi_wvalid
    `ifdef EN_AXI4
        .S_axi4_araddr(S_axi4_araddr), //input [7:0] S_axi4_araddr
		.S_axi4_arburst(S_axi4_arburst), //input [1:0] S_axi4_arburst
		.S_axi4_arcache(S_axi4_arcache), //input [3:0] S_axi4_arcache
		.S_axi4_arid(S_axi4_arid), //input [3:0] S_axi4_arid
		.S_axi4_arlen(S_axi4_arlen), //input [7:0] S_axi4_arlen
		.S_axi4_arlock(S_axi4_arlock), //input S_axi4_arlock
		.S_axi4_arport(S_axi4_arport), //input [2:0] S_axi4_arport
		.S_axi4_arready(S_axi4_arready), //output S_axi4_arready
		.S_axi4_arsize(S_axi4_arsize), //input [2:0] S_axi4_arsize
		.S_axi4_arvalid(S_axi4_arvalid), //input S_axi4_arvalid
		.S_axi4_awaddr(S_axi4_awaddr), //input [7:0] S_axi4_awaddr
		.S_axi4_awburst(S_axi4_awburst), //input [1:0] S_axi4_awburst
		.S_axi4_awcache(S_axi4_awcache), //input [3:0] S_axi4_awcache
		.S_axi4_awid(S_axi4_awid), //input [3:0] S_axi4_awid
		.S_axi4_awlen(S_axi4_awlen), //input [7:0] S_axi4_awlen
		.S_axi4_awlock(S_axi4_awlock), //input S_axi4_awlock
		.S_axi4_awport(S_axi4_awport), //input [2:0] S_axi4_awport
		.S_axi4_awready(S_axi4_awready), //output S_axi4_awready
		.S_axi4_awsize(S_axi4_awsize), //input [2:0] S_axi4_awsize
		.S_axi4_awvalid(S_axi4_awvalid), //input S_axi4_awvalid
		.S_axi4_bid(S_axi4_bid), //output [3:0] S_axi4_bid
		.S_axi4_bready(S_axi4_bready), //input S_axi4_bready
		.S_axi4_bresp(S_axi4_bresp), //output [1:0] S_axi4_bresp
		.S_axi4_bvalid(S_axi4_bvalid), //output S_axi4_bvalid
		.S_axi4_rdata(S_axi4_rdata), //output [31:0] S_axi4_rdata
		.S_axi4_rid(S_axi4_rid), //output [3:0] S_axi4_rid
		.S_axi4_rlast(S_axi4_rlast), //output S_axi4_rlast
		.S_axi4_rready(S_axi4_rready), //input S_axi4_rready
		.S_axi4_rresp(S_axi4_rresp), //output [1:0] S_axi4_rresp
		.S_axi4_rvalid(S_axi4_rvalid), //output S_axi4_rvalid
		.S_axi4_wdata(S_axi4_wdata), //input [31:0] S_axi4_wdata
		.S_axi4_wlast(S_axi4_wlast), //input S_axi4_wlast
		.S_axi4_wready(S_axi4_wready), //output S_axi4_wready
		.S_axi4_wstrb(S_axi4_wstrb), //input [7:0] S_axi4_wstrb
		.S_axi4_wvalid(S_axi4_wvalid), //input S_axi4_wvalid
    `endif
		.Axi_str_rxd_tdata(Axi_str_rxd_tdata), //input [31:0] Axi_str_rxd_tdata
		.Axi_str_rxd_tlast(Axi_str_rxd_tlast), //input Axi_str_rxd_tlast
		.Axi_str_rxd_tready(Axi_str_rxd_tready), //output Axi_str_rxd_tready
		.Axi_str_rxd_tvalid(Axi_str_rxd_tvalid), //input Axi_str_rxd_tvalid
		.Axi_str_txd_tdata(Axi_str_txd_tdata), //output [31:0] Axi_str_txd_tdata
		.Axi_str_txd_tlast(Axi_str_txd_tlast), //output Axi_str_txd_tlast
		.Axi_str_txd_tready(Axi_str_txd_tready), //input Axi_str_txd_tready
		.Axi_str_txd_tvalid(Axi_str_txd_tvalid), //output Axi_str_txd_tvalid
		.Axi_str_txc_tdata(Axi_str_txc_tdata), //output [31:0] Axi_str_txc_tdata
		.Axi_str_txc_tlast(Axi_str_txc_tlast), //output Axi_str_txc_tlast
		.Axi_str_txc_tready(Axi_str_txc_tready), //input Axi_str_txc_tready
		.Axi_str_txc_tvalid(Axi_str_txc_tvalid), //output Axi_str_txc_tvalid
		.Interrupt(), //output Interrupt
		.Mm2s_prmry_reset_out_n(), //output Mm2s_prmry_reset_out_n
		.Mm2s_cntrl_reset_out_n(), //output Mm2s_cntrl_reset_out_n
		.S2mm_prmry_reset_out_n() //output S2mm_prmry_reset_out_n
	);

    AXI_Master  AXI_Master_inst(
        .S_axi_aclk           (S_axi_aclk        ),      
        .hwRstn               (hwRstn            ),         
    `ifdef EN_AXI4
        .S_axi4_araddr        (S_axi4_araddr     ),     
        .S_axi4_arready       (S_axi4_arready    ),     
        .S_axi4_arvalid       (S_axi4_arvalid    ),     
        .S_axi4_awaddr        (S_axi4_awaddr     ),     
        .S_axi4_awready       (S_axi4_awready    ),     
        .S_axi4_awvalid       (S_axi4_awvalid    ),     
        .S_axi4_bready        (S_axi4_bready     ),     
        .S_axi4_bresp         (S_axi4_bresp      ),     
        .S_axi4_bvalid        (S_axi4_bvalid     ),     
        .S_axi4_rdata         (S_axi4_rdata      ),     
        .S_axi4_rlast         (S_axi4_rlast      ),     
        .S_axi4_rready        (S_axi4_rready     ),     
        .S_axi4_rresp         (S_axi4_rresp      ),     
        .S_axi4_rvalid        (S_axi4_rvalid     ),     
        .S_axi4_wdata         (S_axi4_wdata      ),     
        .S_axi4_wlast         (S_axi4_wlast      ),     
        .S_axi4_wready        (S_axi4_wready     ),     
        .S_axi4_wstrb         (S_axi4_wstrb      ),     
        .S_axi4_wvalid        (S_axi4_wvalid     ),     
        .axi4_awaddr_in       (axi4_awaddr_in    ),     
        .axi4_wdata_in        (axi4_wdata_in     ),     
        .axi4_araddr_in       (axi4_araddr_in    ),     
    `endif
        .S_axi_awaddr         (S_axi_awaddr      ),  
        .S_axi_awvalid        (S_axi_awvalid     ),  
        .S_axi_awready        (S_axi_awready     ),  
        .S_axi_wdata          (S_axi_wdata       ),  
        .S_axi_wvalid         (S_axi_wvalid      ),  
        .S_axi_wready         (S_axi_wready      ),  
        .S_axi_bready         (S_axi_bready      ),  
        .S_axi_bresp          (S_axi_bresp       ),  
        .S_axi_bvalid         (S_axi_bvalid      ),  
        .S_axi_araddr         (S_axi_araddr      ),  
        .S_axi_arvalid        (S_axi_arvalid     ),  
        .S_axi_arready        (S_axi_arready     ),  
        .S_axi_rdata          (S_axi_rdata       ),  
        .S_axi_rready         (S_axi_rready      ),  
        .S_axi_rresp          (S_axi_rresp       ),                     
        .S_axi_rvalid         (S_axi_rvalid      )
    );

//------------------------------------//
    always @ (posedge S_axi_aclk) begin
        if (rst_cnt[12:0]>=13'd0 && rst_cnt[12:0]<=13'd1022) begin
            rst_cnt[12:0] <= rst_cnt[12:0] + 'd1;
            hwRstn <= 'b0;
        end else if (rst_cnt[12:0] == 16'd1023) begin
            hwRstn <= 'b1;
        end else begin
            rst_cnt[12:0] <= 16'd0;
            hwRstn <= 'b0;
        end
    end
//stream_slaver
    always @ (posedge S_axi_aclk or negedge hwRstn) begin        //transmit与stream握手
        if (!hwRstn)
            Axi_str_txd_tready <= 'b0;
        else
            if (Axi_str_txd_tvalid && !Axi_str_txd_tready) begin
                Axi_str_txd_tready <= 'b1;
            end else begin
                Axi_str_txd_tready <= 'b0;
            end
    end

    always @ (posedge S_axi_aclk or negedge hwRstn) begin        //receie与stream握手
        if (!hwRstn) begin
            Axi_str_rxd_tvalid <= 'b0;
        end else begin
            Axi_str_txd_tvalid_dly1 <= Axi_str_txd_tvalid;
            Axi_str_txd_tvalid_dly2 <= Axi_str_txd_tvalid_dly1;
            Axi_str_txd_tvalid_dly3 <= Axi_str_txd_tvalid_dly2;
            Axi_str_txd_tvalid_dly4 <= Axi_str_txd_tvalid_dly3;
            Axi_str_txd_tdata_dly1 <= Axi_str_txd_tdata;
            Axi_str_txd_tdata_dly2 <= Axi_str_txd_tdata_dly1;
            Axi_str_txd_tdata_dly3 <= Axi_str_txd_tdata_dly2;
            Axi_str_txd_tlast_dly <= Axi_str_txd_tlast;
            if (Axi_str_txd_tvalid_dly2 && !Axi_str_txd_tvalid_dly3) begin
                Axi_str_rxd_tvalid <= 'b1;
                Axi_str_rxd_tdata <= Axi_str_txd_tdata_dly1;
                Axi_str_rxd_tlast_wait <= Axi_str_txd_tlast_dly;
            end
            if (Axi_str_rxd_tvalid && Axi_str_rxd_tready) begin
                Axi_str_rxd_tvalid <= 'b0;
                Axi_str_rxd_tlast_wait <= 'b0;
                Axi_str_rxd_tdata <= 32'h00000000;
            end
        end
    end
    assign Axi_str_rxd_tlast = Axi_str_rxd_tlast_wait & Axi_str_rxd_tvalid & Axi_str_rxd_tready;
endmodule

module AXI_Master
    (
    input  wire                                 S_axi_aclk                        ,
    input  wire                                 hwRstn                            ,   
    
    `ifdef EN_AXI4
        output   wire[`ADDR_WE_OE-1:0]                      S_axi4_araddr                   ,
        input    wire                                       S_axi4_arready                  ,
        output   reg                                        S_axi4_arvalid                  ,

        output   wire[`ADDR_WE_OE-1:0]                      S_axi4_awaddr                  ,
        input    wire                                       S_axi4_awready                 ,
        output   reg                                        S_axi4_awvalid                 ,

        output reg                                          S_axi4_bready                  ,
        input  wire[1:0]                                    S_axi4_bresp                   ,
        input  wire                                         S_axi4_bvalid                  ,

        input  wire[`FULL_DATAWD-1:0]                       S_axi4_rdata                   ,
        input  wire                                         S_axi4_rlast                   ,
        output reg                                          S_axi4_rready                  ,
        input  wire[1:0]                                    S_axi4_rresp                   ,
        input  wire                                         S_axi4_rvalid                  ,

        output  wire[`FULL_DATAWD-1:0]                      S_axi4_wdata                   ,
        output  reg                                         S_axi4_wlast                   ,
        input   wire                                        S_axi4_wready                  ,
        output  reg[`FULL_DATAWD/8-1:0]                     S_axi4_wstrb                   ,
        output  reg                                         S_axi4_wvalid                  ,
        input  wire[`ADDR_WE_OE-1:0]                        axi4_awaddr_in                 ,
        input  wire[`FULL_DATAWD-1:0]                       axi4_wdata_in                  ,
        input  wire[`ADDR_WE_OE-1:0]                        axi4_araddr_in                 ,
    `endif

    //写地址
    output wire[`ADDR_WE_OE-1:0]                S_axi_awaddr                 ,
    output reg                                  S_axi_awvalid                ,
    input  wire                                 S_axi_awready                ,
    //写数据                                                                    
    output wire[`LITE_DATAWD-1:0]         S_axi_wdata                  ,
    output reg                                  S_axi_wvalid                 ,
    input  wire                                 S_axi_wready                 ,
    //写响应                                                                  ,
    output reg                                  S_axi_bready                 ,
    input  wire[1:0]                            S_axi_bresp                  ,
    input  wire                                 S_axi_bvalid                 ,
    //读地址                                                                    
    output wire[`ADDR_WE_OE-1:0]                S_axi_araddr                 ,
    output reg                                  S_axi_arvalid                ,
    input  wire                                 S_axi_arready                ,
    //读数据                                                                    
    input  wire[`LITE_DATAWD-1:0]         S_axi_rdata                  ,
    output reg                                  S_axi_rready                 ,
    input  wire[1:0]                            S_axi_rresp                  ,                       
    input  wire                                 S_axi_rvalid                 
);
        reg                         WE_frame                         ;
        reg[7:0]                    WE_frame_reg                     ;
        reg                         OE_frame                         ;
        reg[7:0]                    OE_frame_reg                     ;
        reg                         we_wait_flag                     ;
        reg                         we_wait_flag_dly                 ;
        reg                         OE_resp_flag                     ;
        reg                         OE_resp_flag_dly                 ;
        reg[`ADDR_WE_OE-1:0]        axi_awaddr_gener                 ;
        reg[`ADDR_WE_OE-1:0]        axi_araddr_gener                 ;
        reg[`LITE_DATAWD-1:0] axi_wdata_gener                  ;
        reg[`ADDR_WE_OE-1:0]        axi4_awaddr_gener                ;
        reg[`ADDR_WE_OE-1:0]        axi4_araddr_gener                ;
        reg[`FULL_DATAWD-1:0]       axi4_wdata_gener                 ;
        reg[1:0]                    WE_State                         ;
        reg                         we_length_flag                   ;
        reg                         we_data_flag                     ;
        reg[1:0]                    OE_state                         ;
        reg[31:0]                   rlr                              ;
        reg                         OE_end                           ;
        reg                         WE_end                           ;
        reg                         return_WE_flag                   ;
        reg[`ADDR_WE_OE-1:0]        S_axi_araddr_dly                 ;
        reg                         OE_resp_flag_before              ;
        reg[1:0]                    oe_axi4_ctr                      ;
        reg                         AXI4_DSEL_reg   = `AXI4_DATA     ;
        
        
        reg[2:0]       WE_current_state         ;
        reg[2:0]       WE_next_state            ;
        localparam     WE_IDLE =2'b00         ;
        localparam     WE_ADDR =2'b01         ;
        localparam     WE_DATA =2'b11         ;
        localparam     WE_RESP =2'b10         ;
        reg[2:0]       OE_current_state         ;
        reg[2:0]       OE_next_state            ;
        localparam     OE_IDLE =2'b00         ;
        localparam     OE_ADDR =2'b01         ;
        localparam     OE_DATA =2'b11         ;
        localparam     OE_RESP =2'b10         ;
        
        integer                    WE_frame_cnt                     ;
        integer                    OE_frame_cnt                     ;
        integer                    we_wait_cnt                      ;
        integer                    oe_data_wait_cnt                 ;
        integer                    WE_data_cnt                      ;
        integer                    tlr                              ;
        integer                    WE_end_cnt                       ;
        
        integer                    OEaxi_data_cnt                   ;
        integer                    oe_data_cnt                      ;
        integer                    OE_end_cnt                       ;
        
    `ifdef EN_AXI4
        always @ (posedge S_axi_aclk or negedge hwRstn)
            if (!hwRstn)
                S_axi4_wlast<= 'b0;
            else
                S_axi4_wlast<= 'b1;
    `endif
    always @ (posedge S_axi_aclk or negedge hwRstn) begin              
        if (!hwRstn) begin
            WE_frame <= 'b0;
            WE_frame_reg <= 8'h00;
        end else begin
            WE_frame_reg[0] <= WE_frame;
            WE_frame_reg[7:1] <= WE_frame_reg[6:0];
            we_wait_flag_dly <= we_wait_flag;
            if (WE_frame_cnt==0) begin
                WE_frame_cnt <= WE_frame_cnt + 1;
            end else if (WE_frame_cnt>=1 && WE_frame_cnt<=10) begin
                WE_frame_cnt <= WE_frame_cnt + 1;
            end else if (WE_frame_cnt==11) begin
                WE_frame_cnt <= WE_frame_cnt + 1;
                WE_frame <= 'b1;
            end else if (WE_frame_cnt>=12 && WE_frame_cnt<=4095) begin
                WE_frame_cnt <= WE_frame_cnt + 1;
                WE_frame <= 'b0;
            end else begin
                WE_frame_cnt <= 0;
            end
        end
    end
    always @ (posedge S_axi_aclk or negedge hwRstn) begin              
        if (!hwRstn) begin
            OE_frame <= 'b0;
            OE_frame_reg <= 8'h00;
        end else begin
            OE_frame_reg[0] <= OE_frame;
            OE_frame_reg[7:1] <= OE_frame_reg[6:0];
            OE_resp_flag_dly  <= OE_resp_flag;
            if (OE_frame_cnt==0) begin
                OE_frame_cnt <= OE_frame_cnt + 1;
            end else if (OE_frame_cnt>=1 && OE_frame_cnt<=10) begin
                OE_frame_cnt <= OE_frame_cnt + 1;
            end else if (OE_frame_cnt==11) begin
                OE_frame_cnt <= OE_frame_cnt + 1;
                OE_frame <= 'b1;
            end else if (OE_frame_cnt>=12 && OE_frame_cnt<=4095) begin
                OE_frame_cnt <= OE_frame_cnt + 1;
                OE_frame <= 'b0;
            end else begin
                OE_frame_cnt <= 0;
            end
        end
    end
    always @ (posedge S_axi_aclk or negedge hwRstn)
        if (!hwRstn) 
            WE_current_state <= WE_IDLE;
        else
            WE_current_state <= WE_next_state;
            
    always @ (*) begin
        if (!hwRstn) begin
            WE_next_state    = WE_IDLE;
        end else begin
            case (WE_current_state) 
                WE_IDLE:    begin
                                if (WE_frame_reg[6] && !WE_frame_reg[7] && !WE_end) begin
                                    WE_next_state = WE_ADDR;
                                end else begin
                                    WE_next_state = WE_IDLE;
                                end
                            end
                WE_ADDR:    begin
                                `ifdef EN_AXI4
                                    if (S_axi4_awvalid && S_axi4_awready) begin
                                        WE_next_state   = WE_DATA;
                                    end else begin
                                        WE_next_state   = WE_ADDR;
                                    end
                                `else
                                    if ((S_axi_awvalid & S_axi_awready) && (S_axi_wvalid & S_axi_wready)) begin
                                        WE_next_state   = WE_RESP;
                                    end else if ((S_axi_awvalid && S_axi_awready) && !(S_axi_wvalid & S_axi_wready)) begin
                                        WE_next_state   = WE_DATA;
                                    end else if ((S_axi_wvalid & S_axi_wready) && !(S_axi_awvalid & S_axi_awready)) begin
                                        WE_next_state   = WE_IDLE;
                                    end else begin
                                        WE_next_state   = WE_ADDR;
                                    end
                                `endif
                            end
                WE_DATA:    begin
                                `ifdef EN_AXI4
                                    if (S_axi4_wvalid && S_axi4_wready && S_axi4_wlast) begin
                                        WE_next_state   = WE_RESP;
                                    end else begin
                                        WE_next_state   = WE_DATA;
                                    end
                                `else
                                    if (S_axi_wvalid && S_axi_wready) begin
                                        WE_next_state   = WE_RESP;
                                    end else begin
                                        WE_next_state   = WE_DATA;
                                    end
                                `endif
                            end
                WE_RESP:    begin
                                if (!we_wait_flag && we_wait_flag_dly)begin
                                    if (WE_end) begin
                                        WE_next_state = WE_IDLE;
                                    end else begin
                                        WE_next_state = WE_ADDR;
                                    end
                                end else if (return_WE_flag) begin
                                    WE_next_state = WE_ADDR;
                                end else begin
                                    WE_next_state = WE_RESP;
                                end
                            end
                default:    begin WE_next_state = WE_IDLE; end
            endcase
        end
    end

    always @ (posedge S_axi_aclk) begin
        case (WE_current_state) 
            WE_IDLE:    begin
                            S_axi_awvalid   <= 'b0;
                            S_axi_wvalid    <= 'b0;
                            S_axi_bready    <= 'b0;
                            we_wait_flag    <= 'b0;
                            we_wait_cnt     <= 0;
                            return_WE_flag <= 'b0;
                            `ifdef EN_AXI4
                                S_axi4_awvalid   <= 'b0;
                                S_axi4_wvalid    <= 'b0;
                                S_axi4_bready    <= 'b0;
                            `endif
                            we_length_flag   <= 'b0;
                            we_data_flag     <= 'b0;
                        end
            WE_ADDR:    begin
                            `ifdef EN_AXI4
                                if (S_axi4_awready && S_axi4_awvalid) begin
                                    S_axi4_awvalid <= 'b0;
                                    return_WE_flag <= 'b0;
                                end else begin
                                    S_axi4_awvalid <= 'b1;
                                end
                            `else
                                if (S_axi_awready && S_axi_wready) begin
                                    if (S_axi_awvalid && S_axi_wvalid) begin
                                        S_axi_awvalid <= 'b0;
                                        S_axi_wvalid  <= 'b0;
                                        S_axi_bready  <= 'b1;
                                    end else if (S_axi_awvalid && !S_axi_wvalid) begin
                                        S_axi_awvalid <= 'b0;
                                        S_axi_wvalid  <= 'b1;
                                    end else begin
                                        S_axi_awvalid <= 'b1;
                                        S_axi_wvalid  <= 'b1;
                                    end
                                end else if (S_axi_awready && !S_axi_wready) begin
                                    if (S_axi_awvalid) begin
                                        S_axi_awvalid <= 'b0;
                                        S_axi_wvalid  <= 'b1;
                                    end else begin
                                        S_axi_awvalid <= 'b1;
                                        S_axi_wvalid  <= 'b0;
                                    end
                                end else if (!S_axi_awready && S_axi_wready) begin
                                    S_axi_awvalid <= 'b0;
                                    S_axi_wvalid  <= 'b0;
                                end else begin
                                    S_axi_awvalid <= 'b1;
                                    S_axi_wvalid  <= 'b1;
                                end
                            `endif
                        end
            WE_DATA:    begin
                            `ifdef EN_AXI4
                                if (S_axi4_wready && S_axi4_wvalid && !S_axi4_wlast) begin
                                    S_axi4_wvalid <= 'b0;
                                end else if (S_axi4_wready && S_axi4_wvalid && S_axi4_wlast) begin
                                    S_axi4_wvalid <= 'b0;
                                    S_axi4_bready    <= 'b1;
                                end else begin
                                    S_axi4_wvalid <= 'b1;
                                    S_axi4_bready    <= 'b0;
                                end
                            `else
                                S_axi_awvalid <= 'b0;
                                if (S_axi_wready && S_axi_wvalid) begin
                                    S_axi_wvalid <= 'b0;
                                    S_axi_bready    <= 'b1;
                                end else begin
                                    S_axi_wvalid <= 'b1;
                                end
                            `endif
                        end
            WE_RESP:    begin
                            `ifdef EN_AXI4
                                S_axi4_wvalid   <= 'b0;
                                if (S_axi4_bready && S_axi4_bvalid && S_axi4_bresp==2'b00) begin
                                    S_axi4_bready    <= 'b0;
                                    we_length_flag   <= 'b1;
                                end else if (we_length_flag) begin
                                    if ((tlr+1)*4==WE_data_cnt*4) begin
                                        if (S_axi_awready) begin
                                            if (S_axi_awvalid) begin
                                                S_axi_awvalid <= 'b0;
                                                we_length_flag<= 'b0;
                                                we_data_flag  <= 'b1;
                                            end else begin
                                                S_axi_awvalid <= 'b1;
                                            end
                                        end else begin
                                            S_axi_awvalid <= 'b0;
                                        end
                                    end else begin
                                        return_WE_flag <= 'b1;
                                        we_length_flag <= 'b0;
                                    end
                                end else if (we_data_flag) begin
                                    if (S_axi_wready) begin
                                        if (S_axi_wvalid) begin
                                            S_axi_wvalid    <= 'b0;
                                            S_axi_bready    <= 'b1;
                                            we_data_flag    <= 'b0;
                                        end else begin
                                            S_axi_wvalid <= 'b1;
                                        end
                                    end else begin
                                        S_axi_wvalid <= 'b1;
                                    end
                                end else if (S_axi_bready && S_axi_bvalid && S_axi_bresp==2'b00) begin
                                    S_axi_bready    <= 'b0;
                                    we_wait_flag    <= 'b1;
                                end else begin
                                    we_wait_flag    <= we_wait_flag;
                                end
                            `else
                                S_axi_awvalid   <= 'b0;
                                if (S_axi_bready && S_axi_bvalid && S_axi_bresp==2'b00) begin
                                    S_axi_bready    <= 'b0;
                                    we_wait_flag    <= 'b1;
                                end else begin
                                    we_wait_flag    <= we_wait_flag;
                                end
                            `endif
                            if (we_wait_cnt==0 && we_wait_flag) begin
                                we_wait_cnt <= 1;
                            end else if (we_wait_cnt>=1 && we_wait_cnt<=5) begin
                                we_wait_cnt <= we_wait_cnt + 1;
                            end else if (we_wait_cnt==6 && we_wait_flag) begin
                                we_wait_cnt <= 0;
                                we_wait_flag <= 'b0;
                            end else begin
                                we_wait_cnt <= 0;
                            end
                        end
            default:    begin  end
        endcase
    end
    
    always @ (posedge S_axi_aclk or negedge hwRstn)
        if (!hwRstn) 
            OE_current_state <= OE_IDLE;
        else
            OE_current_state <= OE_next_state;
    
    always @ (*) begin
        if (!hwRstn) begin
            OE_next_state    = OE_IDLE;
        end else begin
            case (OE_current_state) 
                OE_IDLE:    begin
                                if (OE_frame_reg[6] && !OE_frame_reg[7]) begin
                                    OE_next_state   = OE_ADDR;
                                end else begin
                                    OE_next_state   = OE_IDLE;
                                end
                            end
                OE_ADDR:    begin
                                `ifdef EN_AXI4
                                    if (S_axi_rvalid && S_axi_rready && S_axi_araddr_dly==8'h24) begin
                                        OE_next_state   = OE_DATA;
                                    end else begin
                                        OE_next_state   = OE_ADDR;
                                    end
                                `else
                                    if (S_axi_arvalid && S_axi_arready) begin
                                        OE_next_state   = OE_DATA;
                                    end else begin
                                        OE_next_state   = OE_ADDR;
                                    end
                                `endif
                            end
                OE_DATA:    begin
                                `ifdef EN_AXI4
                                    if (OE_resp_flag_before) begin
                                        OE_next_state   = OE_RESP;
                                    end else begin
                                        OE_next_state   = OE_DATA;
                                    end
                                `else
                                    if (S_axi_rvalid && S_axi_rready) begin
                                        OE_next_state   = OE_RESP;
                                    end else begin
                                        OE_next_state   = OE_DATA;
                                    end
                                `endif
                            end
                OE_RESP:    begin
                                `ifdef EN_AXI4
                                    if (S_axi4_rresp==2'b00) begin
                                `else
                                    if (S_axi_rresp==2'b00) begin
                                `endif
                                        if (OE_end) begin
                                            OE_next_state   = OE_IDLE;
                                        end else begin
                                            if (!OE_resp_flag && OE_resp_flag_dly) begin
                                                OE_next_state   = OE_ADDR;
                                            end else begin
                                                OE_next_state   = OE_RESP;
                                            end
                                        end
                                    end else begin
                                        OE_next_state   = OE_IDLE;
                                    end
                            end
                default:    begin
                                OE_next_state = OE_IDLE;
                            end
            endcase
        end
    end
    
    always @ (posedge S_axi_aclk) begin
        case (OE_current_state) 
            OE_IDLE:    begin
                            S_axi_arvalid       <= 'b0;
                            S_axi_rready        <= 'b0;
                            OE_resp_flag        <= 'b0;
                            oe_data_wait_cnt    <= 0;
                            oe_axi4_ctr         <= 2'b00;
                            S_axi_araddr_dly    <= `ADDR_WE_OE'd0;
                            oe_data_cnt         <= 0;
                            OE_resp_flag_before  <= 'b0;

                            `ifdef EN_AXI4
                                S_axi4_arvalid      <= 'b0;
                                S_axi4_rready       <= 'b0;
                                rlr                 <= {32{1'b0}};
                            `endif
                        end
            OE_ADDR:    begin
                            `ifdef EN_AXI4
                                case (oe_axi4_ctr)
                                    2'b00:  begin
                                                if (S_axi_arvalid && S_axi_arready) begin
                                                    S_axi_arvalid       <= 'b0;
                                                    oe_axi4_ctr         <= 2'b01;
                                                    S_axi_araddr_dly    <= S_axi_araddr;
                                                end else begin
                                                    S_axi_arvalid       <= 'b1;
                                                end
                                            end
                                    2'b01:  begin
                                                if (S_axi_rvalid && S_axi_rready) begin
                                                    S_axi_rready        <= 'b0;
                                                    oe_axi4_ctr         <= 2'b00;
                                                    rlr                 <= S_axi_rdata[31:0];
                                                    oe_data_cnt         <= 0;
                                                end else begin
                                                    S_axi_rready       <= 'b1;
                                                end
                                            end
                                    default: begin oe_axi4_ctr <= 2'b00; end
                                endcase
                            `else
                                if (S_axi_arvalid && S_axi_arready) begin
                                    S_axi_arvalid       <= 'b0;
                                end else begin
                                    S_axi_arvalid   <= 'b1;
                                end
                            `endif
                        end
            OE_DATA:    begin
                            `ifdef EN_AXI4
                                case (oe_axi4_ctr)
                                    2'b00:  begin
                                                if (S_axi4_arvalid && S_axi4_arready) begin
                                                    S_axi4_arvalid       <= 'b0;
                                                    oe_axi4_ctr          <= 2'b01;
                                                end else begin
                                                    S_axi4_arvalid       <= 'b1;
                                                end
                                            end
                                    2'b01:  begin
                                                if (S_axi4_rvalid && S_axi4_rready) begin
                                                    S_axi4_rready        <= 'b0;
                                                    oe_axi4_ctr          <= 2'b11;
                                                    oe_data_cnt          <= oe_data_cnt + 1;
                                                end else begin
                                                    S_axi4_rready       <= 'b1;
                                                end
                                            end
                                    2'b11:  begin
                                                if ((rlr==oe_data_cnt*4 && !AXI4_DSEL_reg) || (rlr==oe_data_cnt*8 && AXI4_DSEL_reg)) begin
                                                    oe_axi4_ctr <= 2'b10;
                                                end else begin
                                                    oe_axi4_ctr <= 2'b00;
                                                end
                                            end
                                    2'b10:  begin
                                                if (OE_resp_flag_before) begin
                                                    OE_resp_flag_before  <= 'b0;
                                                    oe_axi4_ctr <= 2'b00;
                                                    OE_resp_flag    <= 'b1;
                                                end else begin
                                                    OE_resp_flag_before <= 'b1;
                                                    oe_axi4_ctr <= 2'b10;
                                                end
                                            end
                                    default: begin oe_axi4_ctr <= 2'b00; end
                                endcase
                            `else
                                if (S_axi_rvalid && S_axi_rready) begin
                                    S_axi_rready        <= 'b0;
                                    OE_resp_flag        <= 'b1;
                                end else begin
                                    S_axi_rready   <= 'b1;
                                end
                            `endif
                        end
            OE_RESP:    begin
                            if (OE_resp_flag && oe_data_wait_cnt<=10) begin
                                oe_data_wait_cnt <= oe_data_wait_cnt + 1;
                            end else if (OE_resp_flag && oe_data_wait_cnt==11) begin
                                OE_resp_flag <= 'b0;
                                oe_data_wait_cnt <= 0;
                            end else begin
                                OE_resp_flag <= 'b0;
                                oe_data_wait_cnt <= 0;
                            end
                        end
            default:    begin end
        endcase
    end
    
    `ifdef EN_AXI4
        assign S_axi4_awaddr = (S_axi4_awvalid) ? axi4_awaddr_gener : `ADDR_WE_OE'dz;
        assign S_axi4_wdata  = (S_axi4_wvalid)  ? axi4_wdata_gener  : `FULL_DATAWD'dz;
        assign S_axi4_araddr = (S_axi4_arvalid) ? axi4_araddr_gener : `ADDR_WE_OE'dz;
    `endif
    assign S_axi_awaddr = (S_axi_awvalid) ? axi_awaddr_gener : `ADDR_WE_OE'dz;
    assign S_axi_wdata  = (S_axi_wvalid)  ? axi_wdata_gener  : `FULL_DATAWD'dz;
    assign S_axi_araddr = (S_axi_arvalid) ? axi_araddr_gener : `ADDR_WE_OE'dz;
    
    ///data generate--
    //WE_frame
    
    always @ (posedge S_axi_aclk or negedge hwRstn)
        if (!hwRstn) 
            WE_State <= 2'b00;
        else
            case(WE_State)
                2'b00:  begin
                            
                            WE_end_cnt <= 0;
                            if (WE_frame && !WE_frame_reg[0]) begin
                                WE_State <= 2'b01;
                                WE_end <= 'b0;
                            end else begin
                                WE_State <= 2'b00;
                                WE_data_cnt <= 0;
                                axi_awaddr_gener <= `ADDR_WE_OE'dz;
                                axi_wdata_gener  <= `FULL_DATAWD'dz;
                                tlr <= 0;
                            end
                        end
                2'b01:  begin
                            if (WE_frame_reg[0] && !WE_frame_reg[1]) begin
                                `ifdef EN_AXI4
                                    axi4_awaddr_gener <= `ADDR_WE_OE'd0;
                                    axi4_wdata_gener  <= WE_data_cnt + 1;
                                `else
                                    axi_awaddr_gener <= `ADDR_WE_OE'd16;
                                    axi_wdata_gener  <= WE_data_cnt + 1;
                                `endif
                            end else begin
                                 `ifdef EN_AXI4
                                    if (S_axi4_wready && S_axi4_wvalid) begin
                                        WE_data_cnt  <= WE_data_cnt+1;
                                    end else begin
                                        if ((tlr+1)*4==WE_data_cnt*4) begin
                                            WE_State <= 2'b11;
                                            axi_awaddr_gener <= `ADDR_WE_OE'd20;
                                            if (!AXI4_DSEL_reg) begin
                                                axi_wdata_gener  <= (tlr+1)*4;
                                            end else begin
                                                axi_wdata_gener  <= (tlr+1)*8;
                                            end
                                        end else begin
                                            WE_State <= 2'b01;
                                            axi4_awaddr_gener <= `ADDR_WE_OE'd0;
                                            axi4_wdata_gener  <= WE_data_cnt + 1;
                                        end
                                    end
                                 `else
                                    if (S_axi_wready && S_axi_wvalid) begin
                                        WE_data_cnt  <= WE_data_cnt+1;
                                    end else if (S_axi_bready && S_axi_bvalid) begin
                                        if ((tlr+1)*4==WE_data_cnt*4) begin
                                            WE_State <= 2'b11;
                                            axi_awaddr_gener <= `ADDR_WE_OE'd20;
                                            axi_wdata_gener  <= (tlr+1)*4;
                                        end else begin
                                            WE_State <= 2'b01;
                                        end
                                    end else begin
                                        WE_State <= 2'b01;
                                        axi_awaddr_gener <= `ADDR_WE_OE'd16;
                                        axi_wdata_gener  <= WE_data_cnt + 1;
                                    end
                                `endif
                            end
                        end          
                2'b11:  begin
                            if (S_axi_bready && S_axi_bvalid) begin
                                tlr <= tlr + 1;
                                WE_data_cnt <= 0;
                                if (WE_end) begin
                                    WE_State <= 2'b00;
                                end else begin
                                    WE_State <= 2'b10;
                                    WE_end_cnt <= WE_end_cnt + 1;
                                end
                            end else begin
                                WE_State <= 2'b11;
                            end
                        end
                2'b10:  begin
                            if (WE_end_cnt==`DATA_LENGTH_AND_NUM) begin        //可通过改变T_end_cnt
                                WE_end     <= 'b1;
                                WE_end_cnt <= 0;
                                WE_State <= 2'b00;
                            end else begin
                                WE_State <= 2'b01;
                            end
                        end
                default:    begin end
            endcase
            

    always @ (posedge S_axi_aclk or negedge hwRstn) 
        if (!hwRstn) 
            OE_state <= 2'b00;
        else 
            case(OE_state) 
                2'b00:  begin
                            OEaxi_data_cnt <= 0;
                            OE_end_cnt  <= 0;
                            OE_end      <= 'b0;
                            if (OE_frame) begin
                                OE_state <= 2'b01;
                            end else begin
                                OE_state <= 2'b00;
                            end
                        end
                2'b01:  begin
                            if (OE_frame_reg[0] && !OE_frame_reg[1]) begin
                                OEaxi_data_cnt   <= 0;
                                axi_araddr_gener <= `ADDR_WE_OE'd36;
                            end else begin
                                if (S_axi_rvalid && S_axi_rready) begin
                                    OE_state      <= 2'b11;
                                    
                                    `ifdef EN_AXI4
                                        axi4_araddr_gener <= `ADDR_WE_OE'd0;
                                    `else
                                        axi_araddr_gener <= `ADDR_WE_OE'd32;
                                        rlr <= S_axi_rdata[31:0];
                                    `endif
                                end else begin
                                    OE_state        <= 2'b01;
                                end
                            end
                        end
                2'b11:  begin
                            `ifdef EN_AXI4
                                if (OE_resp_flag_before) begin
                                    OE_state <= 2'b10;
                                    OE_end_cnt <= OE_end_cnt + 1;
                                end else begin
                                    OE_state <= 2'b11;
                                end
                            `else
                                if (S_axi_rvalid && S_axi_rready) begin
                                    OEaxi_data_cnt     <= OEaxi_data_cnt + 1;
                                end else begin
                                    if (OEaxi_data_cnt*4 == rlr && !AXI4_DSEL_reg) begin
                                        rlr <= {32{1'b0}};
                                        OEaxi_data_cnt <= 0;
                                        OE_state <= 2'b10;
                                        OE_end_cnt <= OE_end_cnt + 1;
                                    end else if (OEaxi_data_cnt*8 == rlr && AXI4_DSEL_reg)begin
                                        rlr <= {32{1'b0}};
                                        OEaxi_data_cnt <= 0;
                                        OE_state <= 2'b10;
                                        OE_end_cnt <= OE_end_cnt + 1;
                                    end else begin
                                        OE_state <= 2'b11;
                                    end
                                end
                            `endif
                        end
                2'b10:  begin
                            if (OE_end_cnt==`DATA_LENGTH_AND_NUM) begin
                                OE_end <= 1;
                                OE_end_cnt <= 0;
                            end else begin
                                if (!OE_end) begin
                                    OE_state <= 2'b01;
                                    `ifdef EN_AXI4
                                        axi4_araddr_gener <= `ADDR_WE_OE'd0;
                                    `else
                                        axi_araddr_gener <= `ADDR_WE_OE'd36;
                                    `endif
                                end else begin
                                    OE_state <= 2'b00;
                                end
                            end
                        end
                default:    begin   end
            endcase
endmodule